#include "jngen.h"
#include <vector>

using namespace std;
using namespace jngen;

int main(int argc, char* argv[]) {
    registerGen(argc, argv);
    parseArgs(argc, argv);

    int n = getOpt(0);
    int m = getOpt(1);
    int c = getOpt(2);
    int tree_type = getOpt(3);

    Tree main_tree;
    if (tree_type == 1) {
        main_tree = Tree::random(c);
    } else if (tree_type == 2) {
        main_tree = Tree::bamboo(c);
    } else if (tree_type == 3) {
        main_tree = Tree::binary(c);
    } else if (tree_type == 4) {
        main_tree = Tree::caterpillar(c, c / 2);
    } else if (tree_type == 5) {
        main_tree = Tree::randomKruskal(c);
    } else if (tree_type == 6) {
        main_tree = Tree::star(c);
    } else if (tree_type == 7) {
        main_tree = Tree::randomPrim(c, c / 2);
    }

    vector<int> ns(c, 1);
    vector<int> ms(c);
    int mm = m - (c - 1);
    int nn = n - c;
    while (nn > 0) {
        int ind = rnd.next(c);
        ++ns[ind];
        ++ms[ind];
        --nn;
        --mm;
    }
    if (mm < 0) {
        exit(228);
    }
    while (mm > 0) {
        int ind = rnd.next(c);
        if (ns[ind] > 1) {
            ++ms[ind];
            --mm;
        }
    }

    vector<Graph> grs;
    vector<int> offset(c);
    for (int i = 0; i < c; ++i) {
        grs.push_back(Graph::random(ns[i], ms[i]).allowMulti().connected());
        if (i > 0) {
            offset[i] = offset[i - 1] + ns[i - 1];
        }
    }

    vector<pair<int, int>> edges;
    for (int i = 0; i < c; ++i) {
        for (const auto& e : grs[i].edges()) {
            edges.emplace_back(offset[i] + e.first, offset[i] + e.second);
        }
    }

    for (const auto& e : main_tree.edges()) {
        int u = offset[e.first] + rnd.next(ns[e.first]);
        int v = offset[e.second] + rnd.next(ns[e.second]);
        edges.emplace_back(u, v);
    }

    auto perm = Array::id(n).shuffled();

    cout << n << " " << m << "\n";
    for (auto e : edges) {
        cout << perm[e.first] + 1 << " " << perm[e.second] + 1 << "\n";
    }
}
