#include <iostream>
#include <vector>

using namespace std;

typedef unsigned long long ull;
typedef long long ll;
typedef pair<ull, vector<ull>> factorial;

ull m;
vector<ull> primes;
vector<factorial> facts;

pair<ll, ll> gcd_ext(ll x, ll y) {
    if (x == 0) return {0LL, 1LL};
    pair<ll, ll> coef = gcd_ext(y % x, x);
    return {coef.second - y / x * coef.first, coef.first};
}

ull inv(ull v) {
    static ll mm = m;
    return (gcd_ext(v, m).first % mm + mm) % mm;
}

ull pow(ull x, ull d) {
    if (d == 0) return 1;
    if (d % 2 == 1) return pow(x, d - 1) * x % m;
    ull sqrt = pow(x, d / 2);
    return sqrt * sqrt % m;
}

ull c(unsigned int n, unsigned int k) {
    if (k > n) return 0LL;
    ull value = facts[n].first;
    vector<ull> degs(facts[n].second);

    unsigned int denom[2]{k, n - k};
    for (unsigned int i : denom) {
        for (unsigned int j = 0; j < primes.size(); j++) {
            degs[j] -= facts[i].second[j];
        }
        value = value * inv(facts[i].first) % m;
    }

    for (unsigned int j = 0; j < primes.size(); j++) {
        value = value * pow(primes[j], degs[j]) % m;
    }
    return value;
}

int main() {

    unsigned int k, n;
    cin >> k >> n;
    m = 1000000007;

    ull m2 = m;
    for (ull i = 2; i * i <= m && m2 > 1; i++) {
        if (m2 % i == 0) {
            primes.push_back(i);
            while (m2 % i == 0) {
                m2 /= i;
            }
        }
    }
    if (m2 > 1) {
        primes.push_back(m2);
    }

    facts.emplace_back(1, vector<ull>(primes.size(), 0));
    for (ull i = 1; i <= k; i++) {
        ull value = facts.back().first;
        vector<ull> degs = vector<ull>(facts.back().second);
        ull i2 = i;

        for (unsigned int j = 0; j < primes.size(); j++) {
            while (i2 % primes[j] == 0) {
                degs[j]++;
                i2 /= primes[j];
            }
        }
        value = value * i2 % m;

        facts.emplace_back(value, degs);
    }

    ull answer = 0;
    for (unsigned int t = 0; t <= n / 2; t++) {
        answer = (answer + c(k, t) * c(k - t, n - 2 * t) % m) % m;
    }
    cout << answer << '\n';

}